@@ -2,7 +2,7 @@
  * DFArc main frame
 
  * Copyright (C) 2005, 2006  Dan Walma
- * Copyright (C) 2008, 2010  Sylvain Beucler
+ * Copyright (C) 2008, 2010, 2014  Sylvain Beucler
 
  * This file is part of GNU FreeDink
 
@@ -28,10 +28,10 @@
 #include "DFArcFrame.hpp"
 
 #include <stdio.h>
-#include <fstream>
 
 #include <wx/file.h>
 #include <wx/filename.h>
+#include <wx/filefn.h>
 #include <wx/image.h>
 
 #include <wx/msgdlg.h>
@@ -40,6 +40,8 @@
 #include <wx/listbox.h>
 #include <wx/clntdata.h>
 
+#include <wx/process.h>
+
 #include "IOUtils.hpp"
 #include "InstallVerifyFrame.hpp"
 #include "Options.hpp"
@@ -53,6 +55,51 @@
 #define LOGO_WIDTH 160
 #define LOGO_HEIGHT 120
 
+
+class MonitorDinkExit : public wxProcess
+{
+public:
+  MonitorDinkExit(Config *mConfig) : wxProcess(), mConfig(mConfig) {}
+  virtual void OnTerminate(int pid, int status);
+private:
+  Config *mConfig;
+};
+
+void MonitorDinkExit::OnTerminate(int pid, int status)
+{
+  if (status == -1)
+    ::wxMessageBox(wxString::Format(_("Dink Smallwood ('%s') was not found on your computer."
+				      " Please configure the Dink program name in the Options menu."),
+				    mConfig->mDinkExe.c_str(), wxICON_ERROR, this),
+		   _("Error"));
+  else if (status != 0)
+    ::wxMessageBox(wxString::Format(_("Dink Smallwood failed! Error code %d."),
+  				    status, wxICON_EXCLAMATION, this),
+  		   _("Error"));
+}
+
+
+class MonitorEditorExit : public wxProcess
+{
+public:
+  MonitorEditorExit(Config *mConfig) : wxProcess(), mConfig(mConfig) {}
+  virtual void OnTerminate(int pid, int status);
+private:
+  Config *mConfig;
+};
+
+void MonitorEditorExit::OnTerminate(int pid, int status)
+{
+  if (status == -1)
+    ::wxMessageBox(wxString::Format(_("The editor ('%s') was not found on your computer."
+				      " Please configure the editor program name in the Options menu."),
+				    mConfig->mEditorExe.c_str()),
+		   _("Error"));
+  else if (status != 0)
+    ::wxMessageBox(_("Error while running the editor"), _("Error"));
+}
+
+
 /**
  * Custom paint method to display the animation efficiently.
  * Cf. doc/animation.txt for details.
@@ -88,7 +135,7 @@ public:
     //dst.DrawRectangle(0, 0, LOGO_WIDTH, LOGO_HEIGHT);
     // Draw logo
     wxMemoryDC src(frame->mAllLogos);
-    dst.Blit(0, 0, LOGO_WIDTH, LOGO_HEIGHT, &src, 0, pos);
+    dst.Blit(0, 0, LOGO_WIDTH, LOGO_HEIGHT, &src, 0, rint(pos));
   }
 
   /**
@@ -96,9 +143,8 @@ public:
    */
   void OnIdle(wxIdleEvent &)
   {
-    if (abs(pos - dpos) < 10e-6)
-      // destination reached, using non-exact comparison because pos
-      // and dpos are of type double.
+    if (fabs(pos - dpos) < .5)
+      // destination reached, taking rounding into account.
       return;
 
     // Simple easing
@@ -174,9 +220,9 @@ void DFArcFrame::OnAbout(wxCommandEvent& aCommandEvent)
     "DFArc version %s\n"
     "Copyright (C) 2004  Andrew Reading (merlin)\n"
     "Copyright (C) 2005, 2006  Dan Walma (redink1)\n"
-    "Copyright (C) 2008, 2009, 2010  Sylvain Beucler (Beuc)\n"
+    "Copyright (C) 2008-2014  Sylvain Beucler (Beuc)\n"
     "Build Date: %s\n"
-    "Powered by bzip2 (http://www.bzip.org) and wxWidgets (http://www.wxwidgets.org)"),
+    "Powered by bzip2 (bzip.org) and wxWidgets (wxwidgets.org)"),
 				  version.c_str(), __TDATE__);
   wxMessageBox(wxs, _("About DFArc v3"), wxOK | wxICON_INFORMATION, this);
 }
@@ -224,13 +270,14 @@ void DFArcFrame::PrepareGUI()
 
   // Features not supported by wxGlade:
   SetIcon(wxIcon(dink_xpm));
-  mSplitter->SetSashGravity(0.5); // proportional resize
+  mSplitter->SetSashGravity(0.5); // proportional outer resize
   // Avoid 'unsplit' (where one of the pane is hidden)
   mSplitter->SetMinimumPaneSize(20);
   // Manual contraints
   mDModListBox->SetMinSize(wxSize(150, 165));
   mDmodDescription->SetMinSize(wxSize(-1, 220));
   this->SetMinSize(this->GetBestSize());
+  mSplitter->SetSashPosition(0); // reset to middle pos (wx3 bug?)
   this->SetSize(this->GetMinSize());
   // user can override size restrictions:
   mDmodDescription->SetMinSize(wxSize(-1, -1));
@@ -255,7 +302,6 @@ void DFArcFrame::PrepareGUI()
 
 void DFArcFrame::updateDescription()
 {
-  std::fstream lFileStream;
   wxString lDescription;
 
   // Set the current D-Mod directory to the currently selected item's client string
@@ -328,6 +374,10 @@ void DFArcFrame::SelectDModFromListBox()
 	if (mGameLocaleList->GetSelection() == wxNOT_FOUND)
 	  mGameLocaleList->Select(0);
       }
+#if defined _WIN32 || defined __WIN32__ || defined __CYGWIN__
+      // work-around wx3 bug -> default height=6 despite BestSize=21
+      mGameLocaleList->SetSize(wxSize(-1, mGameLocaleList->GetBestSize().y+1));
+#endif
     }
   ((DFAnimationPanel*)mAnimationPanel)->setIndex(lb_index);
 }
@@ -343,19 +393,14 @@ void DFArcFrame::Install(wxCommandEvent& aCommandEvent)
 {
   wxString description = _("D-Mod files (*.dmod)");
   wxFileDialog FileDlg(0, _("Select a .dmod file"), _T(""), _T(""), description + _T("|*.dmod"),
-        wxOPEN | wxFILE_MUST_EXIST);
+        wxFD_OPEN | wxFD_FILE_MUST_EXIST);
      
   if (FileDlg.ShowModal() == wxID_OK)
     {
-      InstallVerifyFrame *lTemp = new InstallVerifyFrame(FileDlg.GetPath());
-      if (lTemp->ShowModal() == wxID_OK)
-	{
-	  mConfig->mSelectedDmod = lTemp->getSelectedDmod();
-	  
-	  // Update the D-Mod list
-	  refreshDmodList();
-	}
-      lTemp->Destroy();
+      InstallVerifyFrame lTemp(FileDlg.GetPath());
+      lTemp.ShowModal();
+      // Update the D-Mod list
+      refreshDmodList();
     }
 }
 
@@ -449,31 +494,27 @@ void DFArcFrame::OnPlay(wxCommandEvent &Event)
 {
   wxString lCommand = BuildCommand(GAME);
 
-  // Configure locale
-  wxString save_LANGUAGE;
-  wxString save_LC_ALL;
-  wxString save_LC_MESSAGES;
-  wxString save_LC_CTYPE;
-  bool save_LANGUAGE_exists    = wxGetEnv(wxT("LANGUAGE"),    &save_LANGUAGE);
-  bool save_LC_ALL_exists      = wxGetEnv(wxT("LC_ALL"),      &save_LC_ALL);
-  bool save_LC_CTYPE_exists    = wxGetEnv(wxT("LC_CTYPE"),    &save_LC_CTYPE);
-  bool save_LC_MESSAGES_exists = wxGetEnv(wxT("LC_MESSAGES"), &save_LC_MESSAGES);
 
+  // Configure locale
   wxString locale_name = mConfig->mForceLocale;
   int idx = mGameLocaleList->GetSelection();
-  if (idx != wxNOT_FOUND)
+  if (idx != wxNOT_FOUND && mGameLocaleList->HasClientObjectData())
     {
       wxClientData* wcd = mGameLocaleList->GetClientObject(idx);
       if (wcd != NULL)
 	locale_name = ((ClientDataString*)wcd)->s;
     }
+
+  wxEnvVariableHashMap env;
+  wxGetEnvMap(&env);
+
   if (locale_name != wxEmptyString)
     {
       // locale_name is in the form 'xx_YY', e.g. 'da_DK'
 
       // (gettext.info.gz)The LANGUAGE variable: LANGUAGE > LC_ALL > LC_* > LANG
-      wxUnsetEnv(wxT("LANGUAGE"));
-      wxUnsetEnv(wxT("LC_ALL"));
+      env.erase("LANGUAGE");
+      env.erase("LC_ALL");
 
       // Locales need to be precisely set, including the encoding,
       // even if in the case of FreeDink, we don't need it.
@@ -482,14 +523,14 @@ void DFArcFrame::OnPlay(wxCommandEvent &Event)
       // Under woe, things are pretty sloppy, no worry - except that
       // you won't get any warning if you lack fonts or whatever is
       // needed for your language.
-      wxSetEnv(wxT("LC_ALL"), locale_name);
+      env["LC_ALL"] = locale_name;
 #else
       // Look through $(locale -a) and check if there's a matching
       // locale, otherwise explain to the user that he needs to
       // install the locale.  Not sure if there's a similar method
       // under woe to get the installed locales.
       wxArrayString installed_locales;
-      long code = ::wxExecute(wxT("locale -a"), installed_locales, wxEXEC_SYNC);
+      long code = ::wxExecute("locale -a", installed_locales, wxEXEC_SYNC);
       if (code == 0) // good exit code
 	{
 	  bool found = false;
@@ -501,7 +542,7 @@ void DFArcFrame::OnPlay(wxCommandEvent &Event)
 		    {
 		      found = true;
 		      locale_name = installed_locales.Item(i);
-		      wxSetEnv(wxT("LC_ALL"), locale_name);
+		      env["LC_ALL"] = locale_name;
 		      break;
 		    }
 		}
@@ -516,7 +557,7 @@ void DFArcFrame::OnPlay(wxCommandEvent &Event)
 		    {
 		      found = true;
 		      locale_name = installed_locales.Item(i);
-		      wxSetEnv(wxT("LC_ALL"), locale_name);
+		      env["LC_ALL"] = locale_name;
 		      break;
 		    }
 		}
@@ -526,14 +567,14 @@ void DFArcFrame::OnPlay(wxCommandEvent &Event)
 	      // Poor-man's / fallback i18n:
 	      // - LC_CTYPE and LC_MESSAGES to en_US.UTF-8 (if available)
 	      // - LANGUAGE to the locale
-	      found = (installed_locales.Index(wxT("en_US.UTF-8"), false) != wxNOT_FOUND);
+	      found = (installed_locales.Index("en_US.UTF-8", false) != wxNOT_FOUND);
 	      if (!found)
-		found = (installed_locales.Index(wxT("en_US.utf8"), false) != wxNOT_FOUND);
+		found = (installed_locales.Index("en_US.utf8", false) != wxNOT_FOUND);
 	      if (found)
 		{
-		  wxSetEnv(wxT("LC_CTYPE"), wxT("en_US.UTF-8"));
-		  wxSetEnv(wxT("LC_MESSAGES"), wxT("en_US.UTF-8"));
-		  wxSetEnv(wxT("LANGUAGE"), locale_name);
+		  env["LC_CTYPE"] = "en_US.UTF-8";
+		  env["LC_MESSAGES"] = "en_US.UTF-8";
+		  env["LANGUAGE"] = locale_name;
 		}
 	    }
 	  if (!found)
@@ -542,7 +583,7 @@ void DFArcFrame::OnPlay(wxCommandEvent &Event)
 					      " You need to install it - check your system documentation."),
 					    locale_name.c_str()), _("Warning"), wxICON_EXCLAMATION, this);
 	  // try anyway; this also prevent fallback-ing to the system language
-	  wxSetEnv(wxT("LC_ALL"), locale_name);
+	  env["LC_ALL"] = locale_name;
 	}
 #endif
     }
@@ -550,65 +591,26 @@ void DFArcFrame::OnPlay(wxCommandEvent &Event)
 
   // Start the child process.
   long code = 0;
-  bool found = false;
-  bool error = true;
-
-  if (
 #if defined _WIN32 || defined __WIN32__ || defined __CYGWIN__ || defined __EMX__ || defined __DJGPP__
-      (code = ::wxExecute(lCommand, wxEXEC_SYNC)) == -1
-#else
-      // (long)code == 255 if not found, but -1 if segfault, under GNU/Linux
-      (code = ::wxExecute(lCommand, wxEXEC_SYNC)) == 255
+  wxLogNull logNo;  // remove redundant "Execution of command 'xxx' failed" popup
 #endif
-      )
+  wxExecuteEnv exec_env;
+  exec_env.env = env;
+  MonitorDinkExit* process = new MonitorDinkExit(mConfig);
+  if ((code = ::wxExecute(lCommand, wxEXEC_ASYNC, process, &exec_env)) == 0)
     {
-      // Dink executable was not found
-      error = false;
-      found = false;
-    }
-  else if (code != 0)
-    {
-      // Dink returned non-zero or experienced a segmentation fault
-      error = true;
-      found = true;
-    }
-  else
-    {
-      // Dink ran fine
-      error = false;
-      found = true;
-    }
-
-  if (error)
-      ::wxMessageBox(wxString::Format(_("Dink Smallwood failed! Error code %d."),
-				      code, wxICON_EXCLAMATION, this),
+      // On woe, returns immediately if cannot run the process (not using fork&exec)
+      ::wxMessageBox(wxString::Format(_("Dink Smallwood ('%s') was not found on your computer."
+					" Please configure the Dink program name in the Options menu."),
+				      mConfig->mDinkExe.c_str(), wxICON_ERROR, this),
 		     _("Error"));
-  if (!found)
-    ::wxMessageBox(wxString::Format(_("Dink Smallwood ('%s') was not found on your computer."
-				      " Please configure the Dink program name in the Options menu."),
-				    mConfig->mDinkExe.c_str(), wxICON_ERROR, this),
-		   _("Error"));
-
-  // Restore environment variables
-  if (save_LANGUAGE_exists)
-    wxSetEnv(wxT("LANGUAGE"), save_LANGUAGE);
-  else
-    wxUnsetEnv(wxT("LANGUAGE"));
-  if (save_LC_ALL_exists)
-    wxSetEnv(wxT("LC_ALL"), save_LC_ALL);
-  else
-    wxUnsetEnv(wxT("LC_ALL"));
-  if (save_LC_CTYPE_exists)
-    wxSetEnv(wxT("LC_CTYPE"), save_LC_CTYPE);
-  else
-    wxUnsetEnv(wxT("LC_CTYPE"));
-  if (save_LC_MESSAGES_exists)
-    wxSetEnv(wxT("LC_MESSAGES"), save_LC_MESSAGES);
-  else
-    wxUnsetEnv(wxT("LC_MESSAGES"));
-
-  if (mConfig->mCloseDfarcOnPlay && found && !error)
-    this->Close(true);
+      delete process;
+    }
+  else if (mConfig->mCloseDfarcOnPlay)
+    {
+      // since we're async, DFArc will have exited before we know that e.g. Dink couldn't be found
+      this->Close(true);
+    }
 }
 
 void DFArcFrame::onEdit( wxCommandEvent& aEvent )
@@ -637,20 +639,15 @@ void DFArcFrame::onEdit( wxCommandEvent& aEvent )
   // Start the child process.
   long code = 0;
 
-  if (
 #if defined _WIN32 || defined __WIN32__ || defined __CYGWIN__ || defined __EMX__ || defined __DJGPP__
-      (code = ::wxExecute(lCommand, wxEXEC_SYNC)) == -1
-#else
-      // (long)code == 255 if not found, but -1 if segfault, under GNU/Linux
-      (code = ::wxExecute(lCommand, wxEXEC_SYNC)) == 255
+  wxLogNull logNo;  // remove redundant "Execution of command 'xxx' failed" popup
 #endif
-      )
+  MonitorEditorExit* process = new MonitorEditorExit(mConfig);
+  if ((code = ::wxExecute(lCommand, wxEXEC_ASYNC, process)) == 0)
     ::wxMessageBox(wxString::Format(_("The editor ('%s') was not found on your computer."
 				      " Please configure the editor program name in the Options menu."),
 				    mConfig->mEditorExe.c_str()),
 		   _("Error"));
-  else if (code != 0)
-    ::wxMessageBox(_("Error while running the editor"), _("Error"));
 }
 
 void DFArcFrame::onRefresh( wxCommandEvent& aEvent )
@@ -852,14 +849,14 @@ void DFArcFrame::showOptions(wxCommandEvent& aEvent)
 	  if (::wxDirExists(mConfig->mSpecifiedDinkrefDir))
 	    {
 	      // Get the full path to the directory, in case users type in a non-full path.
-	      if (chdir(mConfig->mSpecifiedDinkrefDir.fn_str()) == 0)
+	      if (::wxSetWorkingDirectory(mConfig->mSpecifiedDinkrefDir))
 		{
 		  mConfig->mSpecifiedDinkrefDir = ::wxGetCwd();
 		}
 	      else
 		{
 		  // If there's no directory, let's not override
-		  ::wxMessageBox(_("Cannot use the overriden Dink Smallwood directory"
+		  ::wxMessageBox(_("Cannot use the overridden Dink Smallwood directory"
 				   " - ignoring it. (permission problem?)"),
 				 _("Configuration error"));
 		  mConfig->mOverrideDinkrefDir = false;
@@ -912,7 +909,7 @@ void DFArcFrame::uninstall( wxCommandEvent& aEvent )
                 {
 		  if (::wxRmdir(mConfig->mSelectedDmod) == false)
                     {
-		      ::wxLogError(_("Unable to remove D-Mod directory. All other files were removed."));
+		      wxLogError(_("Unable to remove D-Mod directory. All other files were removed."));
 		      lSuccess = false;
                     }
                 }
